# Copyright 2019 Xanadu Quantum Technologies Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for the recursive_hafnian Python function"""
# pylint: disable=no-self-use,redefined-outer-name
import warnings
import pytest

import numpy as np

from thewalrus import hafnian


@pytest.mark.parametrize("n", [6, 8, 10])
def test_equality(n):
    """Test if recursive_hafnian gives the same as non recursive"""
    A = np.random.rand(n, n)
    A += A.T
    exact = hafnian(A, method="glynn")
    recursive = hafnian(A, method="recursive")
    assert np.allclose(recursive, exact)


def test_recursive_or_loop():
    """Check exception raised if chosen loop and recursive"""
    loop = True
    method = "recursive"
    A = np.random.rand(5, 5)
    A += A.T
    with warnings.catch_warnings(record=True) as w:
        hafnian(A, method=method, loop=loop)

        assert len(w) == 1
        assert "Recursive algorithm does not support the loop hafnian" in str(w[-1].message)
